'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createHookReducers = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _rx = require('rx');

var _rx2 = _interopRequireDefault(_rx);

var _utils = require('../utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var createWrapHookObservable = function createWrapHookObservable(map, pauser) {
  var wrapObservable = function wrapObservable(observable) {
    var memo = map.get(observable);
    if (memo) return memo;
    var wrapped = observable.doOnNext(function () {
      return pauser.onNext(false);
    }).doOnError(function () {
      return pauser.onNext(false);
    }).flatMapObserver(function (val) {
      return _rx2.default.Observable.just(val).doOnCompleted(function () {
        return pauser.onNext(true);
      });
    }, function (err) {
      return _rx2.default.Observable.create(function (o) {
        o.onError(err);
        pauser.onNext(true);
      });
    }).startWith(null).share();
    map.set(observable, wrapped);
    return wrapped;
  };
  return function (action, additionalActionsOrObservables) {
    var actionObservable = (0, _utils.getObservable)(action);
    if (additionalActionsOrObservables.length) {
      var _Rx$Observable;

      var observables = [actionObservable].concat(_toConsumableArray(additionalActionsOrObservables.map(_utils.getObservable))).map(wrapObservable);
      return (_Rx$Observable = _rx2.default.Observable).combineLatest.apply(_Rx$Observable, _toConsumableArray(observables));
    } else {
      return wrapObservable(actionObservable);
    }
  };
};

var createHookReducers = exports.createHookReducers = function createHookReducers(observable, nodeObservable, hookMap, pauser) {
  var allDisposable = new _rx2.default.CompositeDisposable();
  var hookSubject = new _rx2.default.ReplaySubject();
  var nextHandlerSubject = new _rx2.default.ReplaySubject();
  var errorHandlerSubject = new _rx2.default.ReplaySubject();
  var createHookObservable = createWrapHookObservable(hookMap, pauser);
  var createReduce = function createReduce(obs, setNextState, filter) {
    return function (handler) {
      allDisposable.add(obs.filter(function (_ref) {
        var _ref2 = _slicedToArray(_ref, 3);

        var vals = _ref2[0];
        var state = _ref2[1];
        var type = _ref2[2];
        return type === filter;
      }).subscribeOnNext(function (_ref3) {
        var _ref4 = _slicedToArray(_ref3, 2);

        var vals = _ref4[0];
        var state = _ref4[1];

        setNextState(handler(state, vals));
      }));
    };
  };
  var apiRealizationObservable = _rx2.default.Observable.combineLatest(nodeObservable.filter(function (node) {
    return node && !node.provisional;
  }), hookSubject, function (_ref5, hookObservable) {
    var setNextState = _ref5.setNextState;

    var nextSubject = new _rx2.default.Subject();
    var errorSubject = new _rx2.default.Subject();
    var errorObservable = errorSubject.withLatestFrom(observable, _rx2.default.Observable.just('E'));
    var nextObservable = nextSubject.withLatestFrom(observable, _rx2.default.Observable.just('N')).doOnError(function (err) {
      return errorSubject.onNext(err);
    }).onErrorResumeNext(errorObservable);

    allDisposable.add(hookObservable.subscribe(nextSubject));

    nextHandlerSubject.subscribe(createReduce(nextObservable, setNextState, 'N'));
    errorHandlerSubject.subscribe(createReduce(errorObservable, setNextState, 'E'));

    return true;
  });

  apiRealizationObservable.subscribeOnError(function (error) {
    throw error;
  });

  var dispose = function dispose() {
    allDisposable.dispose();
    return api;
  };

  var next = function next(handler) {
    nextHandlerSubject.onNext(handler);
    return makeApi();
  };
  var error = function error(handler) {
    errorHandlerSubject.onNext(handler);
    return makeApi();
  };

  var hookReducers = function hookReducers(action) {
    for (var _len = arguments.length, additionalActionsOrObservables = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      additionalActionsOrObservables[_key - 1] = arguments[_key];
    }

    var hookObservable = createHookObservable(action, additionalActionsOrObservables);

    hookSubject.onNext(hookObservable);
    return makeApi();
  };

  var makeApi = function () {
    var memo = null;
    return function () {
      if (memo) {
        return memo;
      } else {
        return memo = {
          next: next,
          error: error,
          dispose: dispose,
          hookReducers: createHookReducers(observable, nodeObservable, hookMap, pauser)
        };
      }
    };
  }();

  return hookReducers;
};